@@ -144,14 +144,44 @@
 //! // Export to a 16-bit linear light grayscale PNG image.
 //! let png_16bpp_bytes = c.export_image(ImageFormat::PngLinear16Bpp).unwrap();
 //! ```
+//!
+//! Window image export
+//! -------------------
+//!
+//! The `Canvas` object additionally supports windowed image export.
+//!
+//! A single rectangular window represents a region of interest (ROI)
+//! on the canvas image. Window rectangle coordinates are represented
+//! by the public `Window` structure.
+//!
+//! ### Example window image export code
+//!
+//! ```
+//! use planetarium::{Canvas, ImageFormat, Window};
+//!
+//! let mut c = Canvas::new(256, 256);
+//!
+//! c.set_background(1000);
+//! c.clear();
+//!
+//! // Create a 32x16 pixels window with origin at (100, 150).
+//! let wnd = Window::new(32, 16).at(100, 150);
+//!
+//! let fmt = ImageFormat::RawGamma8Bpp;
+//!
+//! // Export to the canvas window image bytes.
+//! let raw_window_bytes = c.export_window_image(wnd, fmt).unwrap();
+//!
+//! // Only the pixels within the window rectangle are exported.
+//! assert_eq!(raw_window_bytes.len(), 32 * 16);
+//! ```
 
 mod draw;
+mod export;
 mod gamma;
 mod pattern;
-mod raw;
 
-#[cfg(feature = "png")]
-mod png;
+pub use crate::export::{EncoderError, ImageFormat, Window, WindowSpans};
 
 use crate::gamma::GammaCurve8;
 use crate::pattern::AiryPattern;
@@ -358,33 +388,6 @@ pub struct Canvas {
     gamma_curve: GammaCurve8,
 }
 
-/// Exportable canvas image formats
-#[derive(Debug, Clone, Copy)]
-#[non_exhaustive]
-pub enum ImageFormat {
-    // Internal encoders:
-    /// 8-bit gamma-compressed grayscale RAW
-    RawGamma8Bpp,
-    /// 10-bit linear light grayscale little-endian RAW
-    RawLinear10BppLE,
-    /// 12-bit linear light grayscale little-endian RAW
-    RawLinear12BppLE,
-
-    // Require "png" feature:
-    /// 8-bit gamma-compressed grayscale PNG
-    PngGamma8Bpp,
-    /// 16-bit linear light grayscale PNG
-    PngLinear16Bpp,
-}
-
-/// Image export encoder error type
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
-#[non_exhaustive]
-pub enum EncoderError {
-    /// Requested image format not supported
-    NotImplemented,
-}
-
 impl Default for SpotShape {
     fn default() -> Self {
         SpotShape {
@@ -490,15 +493,6 @@ impl std::fmt::Display for Transform {
     }
 }
 
-impl std::fmt::Display for EncoderError {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
-        // FIXME: Put full length error descriptions here.
-        write!(f, "{:?}", self)
-    }
-}
-
-impl std::error::Error for EncoderError {}
-
 impl SpotShape {
     /// Linearly scales the spot shape by a single scalar factor.
     pub fn scale(&self, k: f32) -> SpotShape {
@@ -776,29 +770,6 @@ impl Canvas {
     pub fn set_brightness(&mut self, brightness: f32) {
         self.brightness = brightness;
     }
-
-    /// Exports the canvas contents in the requested image format.
-    #[cfg(not(feature = "png"))]
-    pub fn export_image(&self, format: ImageFormat) -> Result<Vec<u8>, EncoderError> {
-        match format {
-            ImageFormat::RawGamma8Bpp => self.export_raw8bpp(),
-            ImageFormat::RawLinear10BppLE => self.export_raw1xbpp::<10>(),
-            ImageFormat::RawLinear12BppLE => self.export_raw1xbpp::<12>(),
-            _ => Err(EncoderError::NotImplemented),
-        }
-    }
-
-    /// Exports the canvas contents in the requested image format.
-    #[cfg(feature = "png")]
-    pub fn export_image(&self, format: ImageFormat) -> Result<Vec<u8>, EncoderError> {
-        match format {
-            ImageFormat::RawGamma8Bpp => self.export_raw8bpp(),
-            ImageFormat::RawLinear10BppLE => self.export_raw1xbpp::<10>(),
-            ImageFormat::RawLinear12BppLE => self.export_raw1xbpp::<12>(),
-            ImageFormat::PngGamma8Bpp => self.export_png8bpp(),
-            ImageFormat::PngLinear16Bpp => self.export_png16bpp(),
-        }
-    }
 }
 
 #[cfg(test)]
